from . import YaTranslateException, _YaBaseAPIHandler, logger


class Dictionary(_YaBaseAPIHandler):
    """
        Implements Yandex Dictionary API methods

        for more info look on https://tech.yandex.com/dictionary/
    """
    _base_url = r"https://dictionary.yandex.net/api/v{version}/" \
                r"dicservice{json}/"
    _endpoints = _YaBaseAPIHandler._endpoints.copy()
    _endpoints.update({
        'lookup': "lookup"
    })

    # filters could be combined throw bitmap arithmetic
    NON_FILTERED = 0  # Remove all filters
    FAMILY = 1  # Apply the family search filter
    MORPHO = 4  # Enable searching by word form
    # Enable a filter that requires matching parts of speech for the search
    # word and translation:
    POS_FILTER = 8

    def __init__(self, api_key: str, xml: bool=False, version: str='1'):
        """
        :param api_key: :type str, personal API access key, given by Yandex
        :param xml: :type bool, specify returned data format
        :param version: :tyep str='1', version of the API
        :param version: :type str=1 – api version

        >>> Dictionary("123")._api_key == "123"  # test for parent method
        True
        >>> Dictionary("123").v == '1'
        True
        >>> Dictionary("123", version='1.5').v == '1.5'
        True
        >>> Dictionary("123")._url
        'https://dictionary.yandex.net/api/v1/dicservice.json/'
        """
        super(Dictionary, self).__init__(api_key, xml, version)
        self._url = self._base_url.format(version=self._v, json=self._json)

    def get_langs(self, **params) -> ...:
        """
        Wrapper for getLangs API method. Use caching to store received info.
        https://tech.yandex.com/dictionary/doc/dg/reference/getLangs-docpage/

        :param params: supported additional params: callback, proxies, update
        :return: collection of supported languages
        """
        return super(Dictionary, self)._get_langs(self._url, **params)

    def getLangs(self, **params) -> ...:
        return self.get_langs(**params)

    @property
    def ok(self) -> bool:
        """
        To check that the API key is correct.

        :return: :type bool
        """
        return super(Dictionary, self)._ok(self._url)

    def lookup(self, text: str, lang: str, ui: str='en', flags: int=0,
               post: bool=False, **parameters) -> ...:
        """
        Wrapper for lookup API method.
        https://tech.yandex.com/dictionary/doc/dg/reference/lookup-docpage/

        Attributes:
        def â€” array of dictionary entries
        ts â€” a transcription of the search word
        tr â€” array of translations
        syn â€” array of synonyms
        mean â€” array of meanings
        ex â€” array of examples

        Subattributes:
        text â€” text of the entry, translation, or synonym (mandatory)
        pos â€” part of speech (may be omitted)

        :param text: :type str, the word or phrase to find in the dictionary
        :param lang: :type str, translation direction (set as a pair of language codes separated by a hyphen)
        :param ui: :type str='en', the language of the user's interface for displaying names of parts of speech in the dictionary entry
        :param flags: :type int=0, search options (bitmask of flags)
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type dict or ElementTree.Element or requests.Response
        :exception YaTranslateException, ConnectionError from requests.exceptions
        """
        if lang not in self.get_langs():
            raise YaTranslateException(501)
        params = super(Dictionary, self)._form_params(
            text=text,
            lang=lang,
            ui=ui,
            flags=flags,
            **parameters
        )
        response = super(Dictionary, self).make_combined_request(
            "lookup", post, **params
        )
        if self._json:
            del response['head']  # depreciated attribute
        return response

    def definitions(self, text: str, lang: str, **params) -> ...:
        """
        Shortcut for lookup(...)['def'].
        Return array of dictionary entries.
        A transcription of the search word may be provided in the 'ts' attribute.

        :param text: :type str, the word or phrase to find in the dictionary
        :param lang: :type str, translation direction (set as a pair of language codes separated by a hyphen)
        :param params: dict of additional params
        :return list, None or NotImplemented
        :exception YaTranslateException, ConnectionError from requests.exceptions, ValueError

        >>> Dictionary("123", xml=True).definitions("hello", "en-en")
        NotImplemented
        >>> Dictionary("123").definitions("hello", "en-en", callback=abs)
        Traceback (most recent call last):
            ...
        ValueError: definitions() Wrong usage of callback
        """
        if "callback" in params:
            raise ValueError("Wrong usage of callback")
        elif not self._json:
            return NotImplemented
        return self.lookup(text, lang, **params).get("def", None)


class Speller(_YaBaseAPIHandler):
    """
        Implements Yandex Speller API methods

        for more info look on https://tech.yandex.ru/speller/
        no SOAP interface available
    """
    _base_url = r"http://speller.yandex.net/services/spellservice{json}/"
    _endpoints = {
        'text': "checkText",
        'texts': "checkTexts"
    }

    # options could be combined throw arithmetic (not bitmap!)
    IGNORE_UPPERCASE = 1  # (e.g. "DOTA")
    IGNORE_DIGITS = 2
    IGNORE_URLS = 4
    FIND_REPEAT_WORDS = 8  # (e.g. "What _is is_ going to be then?")
    IGNORE_LATIN = 16
    NO_SUGGEST = 32  # only check text without correction suggestions
    FLAG_LATIN = 128
    BY_WORDS = 256  # don't use context
    IGNORE_CAPITALIZATION = 512  # ignore wrong capitalization (e.g. "aLeX")
    IGNORE_ROMAN_NUMERALS = 2048

    # error codes
    ERROR_UNKNOWN_WORD = 1  # there is no such word in dictionary
    ERROR_REPEAT_WORD = 2
    ERROR_CAPITALIZATION = 3
    ERROR_TOO_MANY_ERRORS = 4

    def __init__(self, xml: bool=False, encoding: str='utf-8', **kwargs):
        """
        :param xml: :type bool, specify returned data format (json/xml)
        :param encoding: :type str='utf-8', possible variants: 'utf-8', '1251'

        >>> Speller()._api_key == '_'  # test for parent method
        True
        >>> Speller()._ie == "utf-8"
        True
        >>> Speller(encoding="1251")._ie == "1251"
        True
        >>> Speller(encoding="wubbalubbadubdub")
        Traceback (most recent call last):
            ...
        ValueError: Wrong encoding: wubbalubbadubdub
        """
        super(Speller, self).__init__(kwargs.get('api_key', '_'), xml)
        if encoding.lower() not in self.encodings:
            raise ValueError("Wrong encoding: {}".format(encoding.lower()))
        self._ie = encoding.lower()
        self._url = self._base_url.format(json=self._json)

    def get_langs(self) -> set:
        """
        List of supported by service languages

        :return: :type set, collections of supported languages
        """
        return {'ru', 'en', 'ua'}

    def getLangs(self) -> set:
        return self.get_langs()

    @property
    def encodings(self) -> set:
        """
        Collection of supported encodings
        """
        return {"utf-8", "1251"}

    @property
    def ok(self) -> bool:
        """
        No need to test api key.

        :return: :type bool
        """
        return super(Speller, self)._ok(None, self.check_text, "hello")

    def _check(self, endpoint: str, text: str or list, lang: list=["ru", "en"],
               options: int=0, fmt: str="plain", post: bool=False,
               **parameters) -> ...:
        """
        Wrapper for getText and getTexts API methods.
        https://tech.yandex.ru/speller/doc/dg/reference/checkText-docpage/
        https://tech.yandex.ru/speller/doc/dg/reference/checkTexts-docpage/

        Attributes:
        error – info about error (or errors, each object is separate error)
        word – original word
        s – hint (could be None, one or many)
        code – error code
        pos – position of incorrect word
        row – row of error
        col – number or column with error
        len – length of incorrect word

        :param endpoint: :type str, method endpont
        :param text: :type str or list, the word or phrase to check
        :param lang: :type list, supported correction languages (possible values in Speller._languages)
        :param options: :type int, configurations for speller checker (flags)
        :param fmt: :type str, text format ('plain' or 'html')
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type list or ElementTree.Element or requests.Response
        :exception YaTranslateException, ValueError, ConnectionError from requests.exceptions
        """
        if endpoint not in self._endpoints:
            raise ValueError("wrong endpoint {}".format(endpoint))
        if post:
            return NotImplemented
        for lng in lang:
            if lng not in self.get_langs():
                raise YaTranslateException(501)
        params = super(Speller, self)._form_params(
            text=text,
            lang=",".join(lang),
            options=options,
            format=fmt,
            ie=self._ie,
            **parameters
        )
        return super(Speller, self).make_combined_request(
            endpoint, post=False, **params
        )

    def check_text(self, text: str, lang: list=["ru", "en"], options: int=0,
                   fmt: str="plain", post: bool=False, **parameters) -> ...:
        """
        Wrapper for getText API method.
        https://tech.yandex.ru/speller/doc/dg/reference/checkText-docpage/

        :param text: :type str, the word or phrase to check
        :param lang: :type list=['ru', 'en'], supported correction languages (possible values in Speller._languages)
        :param options: :type int=0, configurations for speller checker (flags)
        :param fmt: :type str="plain", text format ('plain' or 'html')
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type list or ElementTree.Element or requests.Response
        :exception YaTranslateException, ValueError, ConnectionError from requests.exceptions
        """
        return self._check(
            endpoint="text",
            text=text,
            lang=lang,
            options=options,
            fmt=fmt,
            post=post,
            **parameters
        )

    def checkText(self, text: str, **params) -> ...:
        return self.check_text(text, **params)

    def check_texts(self, text: list, lang: list=["ru", "en"], options: int=0,
                    fmt: str="plain", post: bool=False, **parameters) -> ...:
        """
        Wrapper for getTexts API method.
        https://tech.yandex.ru/speller/doc/dg/reference/checkTexts-docpage/

        :param text: :type list, the word or phrase to check
        :param lang: :type list=['ru', 'en'], supported correction languages (possible values in Speller._languages)
        :param options: :type int=0, configurations for speller checker (flags)
        :param fmt: :type str="plain", text format ('plain' or 'html')
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type list or xml.etree.ElementTree.ElementTree or requests.Response
        :exception YaTranslateException, ValueError, ConnectionError from requests.exceptions
        """
        return self._check(
            endpoint="texts",
            text=text,
            lang=lang,
            options=options,
            fmt=fmt,
            post=post,
            **parameters
        )

    def checkTexts(self, text: list, **params) -> ...:
        return self.check_texts(text, **params)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



from . import _YaBaseAPIHandler


class Translator(_YaBaseAPIHandler):
    """
        Implements all Yandex Translator API methods

        for more info look on https://tech.yandex.com/translate/
    """
    _base_url = r"https://translate.yandex.net/api/v{version}/tr{json}/"
    _endpoints = _YaBaseAPIHandler._endpoints.copy()
    _endpoints.update({
        'detect': "detect",
        'translate': "translate"
    })

    def __init__(self, api_key: str, xml: bool=False, version: str='1.5'):
        """
        :param api_key: :type str, personal API access key, given by Yandex
        :param xml: :type bool, specify returned data format
        :param version: :tyep str='1.5', version of the API
        :param kwargs: :type str='1.5' – api version

        >>> Translator("123")._api_key == "123"  # test for parent method
        True
        >>> Translator("123").v == '1.5'
        True
        >>> Translator("123", version='1').v == '1'
        True
        >>> Translator("123")._url
        'https://translate.yandex.net/api/v1.5/tr.json/'
        """
        super(Translator, self).__init__(api_key, xml, version)
        self._url = self._base_url.format(version=self._v, json=self._json)

    def get_langs(self, lang: str='en', **params) -> ...:
        """
        Wrapper for getLangs API method. Use caching to store received info.
        https://tech.yandex.com/translate/doc/dg/reference/getLangs-docpage/

        :param lang: :type str='en', language names are output in the language corresponding to the code in this parameter
        :param params: supported additional params: callback, proxies, update
        :return: :type dict or list or ElementTree.Element or requests.Response
        """
        return super(Translator, self)._get_langs(self._url, ui=lang, **params)

    def getLangs(self, lang: str='en', **params) -> ...:
        return self.get_langs(lang, **params)

    @property
    def directions(self) -> list or None:
        """
        Shortcut for get_langs(...)['dirs'].

        :return: :type list or None, supported directions of translation
        """
        if not self._json:
            return self.get_langs()
        return self.get_langs().get('dirs', None)

    @property
    def languages(self) -> list or None or NotImplemented:
        """
        Shortcut for get_langs(...)['langs'].

        :return: :type dict or None or NotImplemented, supported languages
        """
        if not self._json:
            return NotImplemented
        return self.get_langs().get('langs', None)

    @property
    def ok(self) -> bool:
        """
        To check that the API key is correct.

        :return: :type bool
        """
        return super(Translator, self)._ok(self._url)

    def detect(self, text: str, hint: list=None, post: bool=False,
               **parameters) -> ...:
        """
        Wrapper for detect API method.
        https://tech.yandex.com/translate/doc/dg/reference/detect-docpage/

        :param text: :type str, text to detect the language for
        :param hint: :type list=None, list of the most likely languages (they will be given preference when detecting the text language)
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type str or ElementTree.Element or requests.Response
        :exception ValueError
        """
        if hint and not isinstance(hint, list):
            raise ValueError("'hint' should be type {}".format(type(list)))
        params = super(Translator, self)._form_params(
            text=text,
            hint=hint,
            **parameters
        )
        response = super(Translator, self).make_combined_request(
            "detect", post, **params
        )
        if self._json:
            return response['lang']
        return response

    def translate(self, text: str or list, language: str,
                  formatting: str="plain", options: int=1, post: bool=False,
                  **parameters) -> ...:
        """
        Wrapper for translate API method.
        https://tech.yandex.com/translate/doc/dg/reference/translate-docpage/

        :param text: :type str or list of str, text to translate
        :param language: :type str, the translation direction (or just the target language code)
        :param formatting: :type str='plain', text format ('plain' or 'html')
        :param options: :type int=1, the only option available at this time is whether the response should include the automatically detected language of the text being translated (options=1)
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type dict or ElementTree.Element or requests.Response
        """
        params = super(Translator, self)._form_params(
            text=text,
            lang=language,
            format=formatting,
            options=options,
            **parameters
        )
        response = super(Translator, self).make_combined_request(
            "translate", post, **params
        )
        if self._json:
            del response['code']  # this information is redundant
        return response


if __name__ == "__main__":
    import doctest
    doctest.testmod()



"""
    This module implements some Yandex Linguistics APIs for Python 3.3+

    Powered by Yandex (https://tech.yandex.com)
"""

from .baseTypes import YaTranslateException, _YaBaseAPIHandler
from .config import logger


def Translator(api_key: str, xml: bool=False, version: str='1.5'):
    from .Translate import Translator
    return Translator(api_key=api_key, xml=xml, version=version)


def Dictionary(api_key: str, xml: bool=False, version: str='1'):
    from .Vocabulary import Dictionary
    return Dictionary(api_key=api_key, xml=xml, version=version)


def Predictor(api_key: str, xml: bool=False, version: str='1'):
    from .Prediction import Predictor
    return Predictor(api_key=api_key, xml=xml, version=version)


def Speller(xml: bool=False, encoding: str='utf-8', **kwargs):
    from .Vocabulary import Speller
    return Speller(xml=xml, encoding=encoding, **kwargs)


__all__ = ["Dictionary", "Translator", "YaTranslateException",
           "Predictor", "Speller"]



import http
import json
from collections import Callable
from urllib import parse, request
from xml.etree import ElementTree

from .config import logger


class YaTranslateException(Exception):
    """
        Class for possible exceptions during Yandex API usage
    """
    error_codes = {
        400: "Wrong parameter was specified",
        401: "Invalid API key",
        402: "Blocked API key",
        403: "Exceeded the daily limit on the amount of requests",
        404: "Exceeded the daily limit on the amount of translated text",
        413: "Exceeded the maximum text size",
        422: "The text cannot be translated",
        501: "The specified translation direction is not supported",
        503: "Server not available",
    }

    def __init__(self, status_code: int, *args, **kwargs):
        """
        :param status_code: :type int, code, returned from request to API

        >>> YaTranslateException()
        Traceback (most recent call last):
            ...
        TypeError: __init__() missing 1 required positional argument: 'status_code'
        >>> YaTranslateException(400)
        YaTranslateException('Wrong parameter was specified', 400)
        >>> YaTranslateException(300)
        YaTranslateException('Unknown code', 300)
        >>> isinstance(YaTranslateException(400).error_codes, dict)
        True
        """
        message = self.error_codes.get(status_code, "Unknown code")
        super(YaTranslateException, self).__init__(message, status_code,
                                                   *args, **kwargs)


class _YaBaseAPIHandler:
    """
        Generic class for Yandex APIs
    """
    _base_url = r""  # base api url for all requests
    _endpoints = {
        'langs': "getLangs"
    }

    def __init__(self, api_key: str, xml: bool=False, version: str=None):
        """
        :param api_key: :type str, personal API access key, given by Yandex
        :param xml: :type bool, specify returned data format

        >>> _YaBaseAPIHandler()
        Traceback (most recent call last):
            ...
        TypeError: __init__() missing 1 required positional argument: 'api_key'
        >>> _YaBaseAPIHandler('')
        Traceback (most recent call last):
            ...
        YaTranslateException: ('Invalid API key', 401)
        >>> _YaBaseAPIHandler("123")._api_key == "123"
        True
        >>> _YaBaseAPIHandler("123")._json == ".json"
        True
        >>> not _YaBaseAPIHandler("123")._url
        True
        """
        if not api_key:
            raise YaTranslateException(401)
        self._api_key = api_key
        self._json = ".json" if not xml else ""
        self._cache_langs = None
        self._v = version
        self._url = self._base_url

    @property
    def v(self) -> str or NotImplemented:
        if self._v:
            return self._v
        return NotImplemented

    def _make_url(self, endpoint: str, url: str=None) -> str:
        """
        Creates full (base url + endpoint) url for API requests.

        :param endpoint: :type str, one of the keys from self._endpoints dict
        :param url: :type str=None, base url, if not self._url should be used
        :return: :type str, created url for requests

        >>> _YaBaseAPIHandler("123")._make_url('')
        Traceback (most recent call last):
            ...
        KeyError: ''
        >>> _YaBaseAPIHandler("123")._make_url('', "http://url.../") == "http://url.../"
        True
        """
        # not format string for Python less then 3.6 compatibility
        if url:
            return "{}{}".format(url, self._endpoints[endpoint])
        return "{}{}".format(self._url, self._endpoints[endpoint])

    def _form_params(self, **params) -> dict:
        """
        Returns dict of params for request, including API key and etc.

        :param params: :type dict, dict of named params
        :return: :type dict, formed params

        >>> _YaBaseAPIHandler("123")._form_params()
        {'key': '123'}
        >>> 'ui' in _YaBaseAPIHandler("123")._form_params(ui='en')
        True
        >>> 'callback' in _YaBaseAPIHandler("123")._form_params(abs)
        True
        >>> 'callback' not in _YaBaseAPIHandler("123", xml=True)._form_params(abs)
        True
        """
        for key in params:
            if isinstance(params[key], list):
                params[key] = ",".join(params[key])
        parameters = {key: params[key] for key in params
                      if params[key] is not None}
        parameters['key'] = self._api_key
        # for JSONB response
        if 'callback' in parameters and not self._json:
            del parameters['callback']
        return parameters

    def _get_langs(self, base_url: str, update: bool=False, **params) -> ...:
        """
        Wrapper for getLangs API methods.
        Use caching to store received info.

        :param base_url: :type str, base url for requests
        :param update: :type bool=False, update caching values
        :param params: supported additional params: callback, proxies
        :return: :type dict or list or xml.etree.ElementTree.ElementTree or requests.Response
        """
        parameters = {'post': False}
        parameters.update(**self._form_params(**params))
        if "callback" in params:
            return self._make_request(url=self._make_url("langs", base_url),
                                      **parameters)
        if update or not self._cache_langs:
            response = self.make_combined_request(endpoint="langs",
                                                  **parameters)
            if self._json:
                self._cache_langs = response
            elif response.find('dirs'):
                self._cache_langs = [direction.text for direction
                                     in response.find('dirs')]
            else:
                self._cache_langs = [lang.text for lang in response]
        return self._cache_langs

    @staticmethod
    def _make_request_xml(url: str, post: bool=False,
                          **params) -> ElementTree.ElementTree:
        """
        Implements request to API with given params and return content in XML.

        :param url: :type str, prepared url for request to API
        :param post: :type bool, key for making POST request instead GET
        :param params: additional params for request, transmitted throw 'params'
        :return: :type xml.etree.ElementTree.ElementTree, response XML content
        :exception YaTranslateException, requests.exceptions.ConnectionError
        """
        response = _YaBaseAPIHandler._make_request(url, post, **params)
        return ElementTree.fromstring(response.read().decode('utf-8'))

    @staticmethod
    def _make_request_json(url: str, post: bool=False,
                           **params) -> dict or list:
        """
        Implements request to API with given params and return content in JSON.

        :param url: :type str, prepared url for request to API
        :param post: :type bool, key for making POST request instead GET
        :param params: additional params for request, transmitted throw 'params'
        :return: :type dict or list, response content in JSON (response.json())
        :exception YaTranslateException, requests.exceptions.ConnectionError
        """
        response = _YaBaseAPIHandler._make_request(url, post, **params)
        return json.loads(response.read().decode('utf-8'))

    @staticmethod
    def _make_request(url: str, post: bool=False,
                      **params) -> http.client.HTTPResponse:
        """
        Implements request to API with given params.

        :param url: :type str, prepared url for request to API
        :param post: :type bool, key for making POST request instead GET
        :param params: additional params for request, transmitted throw 'params'
        :return: :type request.Response
        :exception YaTranslateException, requests.exceptions.ConnectionError
        """
        url_params = parse.urlencode(params)
        if not post:
            full_url = "{}?{}".format(url, url_params)
            response = request.urlopen(full_url)
        else:
            response = request.urlopen(url, data=url_params.encode('utf-8'))
        if response.code != 200:
            raise YaTranslateException(response.code)
        return response

    def make_combined_request(self, endpoint: str, post: bool=False,
                              **params) -> ...:
        """
        Handle JSON, JSONB and XML requests to API with given params.

        :param endpoint: :type str, one of the keys from self._endpoints dict
        :param post: :type bool, key for making POST request instead GET
        :param params: additional params for request, transmitted throw 'params'
        :return: :type dict or list or request.Response or ElementTree
        :exception YaTranslateException, requests.exceptions.ConnectionError
        """
        parameters = {
            'url': self._make_url(endpoint),
            'post': post
        }
        parameters.update(params)
        if "callback" in params:
            return self._make_request(**parameters)
        elif not self._json:
            return self._make_request_xml(**parameters)
        return self._make_request_json(**parameters)

    def _ok(self, url: str=None, func: Callable=None, *args, **params) -> bool:
        """
        To check that the API key is correct.

        :param url: :type str=None, base url for self._get_langs call
        :param func: :type Callable=None, function to call to test api key
        :param args: :type list, params to pass in called functions
        :param params: :type dict, params to pass in called functions
        :return: :type bool
        """
        try:
            if func:
                __ = func(*args, **params)
            else:
                __ = self._get_langs(url, update=True, *args, **params)
        except http.client.HTTPException as err:
            logger.warning(err)
            return False
        except YaTranslateException as err:
            logger.warning(err)
            return False
        return True


__all__ = ["YaTranslateException"]


if __name__ == "__main__":
    import doctest
    doctest.testmod()



from . import YaTranslateException, _YaBaseAPIHandler


class Predictor(_YaBaseAPIHandler):
    """
        Implements all Yandex Predictor API methods

        for more info look on https://tech.yandex.ru/predictor/
    """
    _base_url = r"https://predictor.yandex.net/api/v{version}/predict{json}/"
    _endpoints = _YaBaseAPIHandler._endpoints.copy()
    _endpoints.update({
        'complete': "complete"
    })

    def __init__(self, api_key: str, xml: bool=False, version: str='1'):
        """
        :param api_key: :type str, personal API access key, given by Yandex
        :param xml: :type bool, specify returned data format
        :param version: :tyep str='1', version of the API
        :param version: :type str='1' – api version

        >>> Predictor("123")._api_key == "123"  # test for parent method
        True
        >>> Predictor("123").v == '1'
        True
        >>> Predictor("123", version='1.5').v == '1.5'
        True
        >>> Predictor("123")._url
        'https://predictor.yandex.net/api/v1/predict.json/'
        """
        super(Predictor, self).__init__(api_key, xml, version)
        self._url = self._base_url.format(version=self._v, json=self._json)

    def get_langs(self, **params) -> ...:
        """
        Wrapper for getLangs API method. Use caching to store received info.
        https://tech.yandex.ru/predictor/doc/dg/reference/getLangs-docpage/

        :param params: supported additional params: callback, proxies, update
        :return: collection of supported languages
        """
        return super(Predictor, self)._get_langs(self._url, **params)

    def getLangs(self, **params) -> ...:
        return self.get_langs(**params)

    @property
    def ok(self) -> bool:
        """
        To check that the API key is correct.

        :return: :type bool
        """
        return super(Predictor, self)._ok(self._url)

    def complete(self, lang: str, q: str, limit: int=1, post: bool=False,
                 **parameters) -> ...:
        """
        Wrapper for complete API method.
        https://tech.yandex.ru/predictor/doc/dg/reference/complete-docpage/

        :param lang: :type str, text language
        :param q: :type str, text under user's pointer
        :param limit: :type int=1, max number of returned strings
        :param post: :type bool=False, key for making POST request instead GET
        :param parameters: supported additional params: callback, proxies
        :return: :type dict or ElementTree.Element or requests.Response
        :exception YaTranslateException
        """
        if lang not in self.get_langs():
            raise YaTranslateException(501)
        params = super(Predictor, self)._form_params(
            lang=lang,
            q=q,
            limit=limit,
            **parameters
        )
        return super(Predictor, self).make_combined_request(
            "complete", post, **params
        )


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
try:
    from setuptools import setup, find_packages
    packages = find_packages(exclude=['tests'])
except ImportError:
    from distutils.core import setup
    packages = ["pyLinguist"]

setup(
    name="pyLinguist",
    packages=packages,
    version="0.1.3rc1",
    description="Yandex Linguistics APIs for Python 3.3+",
    long_description="Yandex Linguistics APIs for Python 3.3+",
    author="hell03end",
    author_email="hell03end@outlook.com",
    url="https://github.com/hell03end/pyYandexLinguistics",
    keywords="translate dictionary predict predictor yandex linguistics "
             "yandex-translate yandex-dictionary yandex-predictor speller"
             "spelling grammar yandex-speller",
    classifiers=[
        "Intended Audience :: Developers",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.3",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    license="MIT License",
    platforms=["All"],
    python_requires=">=3.3"
)
